//*****************************************************************************
//
//  Author:         Kenny Kerr
//  Date created:   31 December 2005
//
//  Description:    Provides common definitions used for diagnostics.
//
//*****************************************************************************

#pragma once

#define ASSERT(x) ATLASSERT(x)
#define TRACE ATLTRACE

#if defined _DEBUG
#define VERIFY(x) ASSERT(x)
#else
#define VERIFY(x) ((void)(x))
#endif

#if defined _DEBUG
#define COM_VERIFY(x) ASSERT(SUCCEEDED(x))
#else
#define COM_VERIFY(x) ((void)(x))
#endif

#if defined _DEBUG
#define GDIPLUS_VERIFY(x) ASSERT(Kerr::Succeeded(x))
#else
#define GDIPLUS_VERIFY(x) ((void)(x))
#endif

namespace Kerr
{
    inline bool Succeeded(Gdiplus::Status errorCode)
    {
        return Gdiplus::Ok == errorCode;
    }

    template <typename T>
    static bool Succeeded(const T& object)
    {
        return Succeeded(object.GetLastStatus());
    }

    template <typename T>
    static bool Succeeded(const std::auto_ptr<T>& object)
    {
        ASSERT(0 != object.get());
        return Succeeded(*object);
    }
}
