//*****************************************************************************
//
//  Author:         Kenny Kerr
//  Date created:   6 August 2006
//
//  Description:    Provides a number of helper functions for dealing with the 
//                  Desktop Window Manager.
//
//*****************************************************************************

#pragma once

namespace Kerr
{
    inline bool IsCompositionEnabled()
    {
        HMODULE library = ::LoadLibrary(L"dwmapi.dll");
        bool result = false;

        if (0 != library)
        {
            if (0 != ::GetProcAddress(library, 
                                      "DwmIsCompositionEnabled"))
            {
                BOOL enabled = FALSE;
                result = SUCCEEDED(::DwmIsCompositionEnabled(&enabled)) && enabled;
            }

            VERIFY(::FreeLibrary(library));
        }

        return result;
    }

    inline HRESULT EnableBlurBehindWindow(HWND window,
                                          bool enable = true,
                                          HRGN region = 0,
                                          bool transitionOnMaximized = false)
    {
        DWM_BLURBEHIND blurBehind = { 0 };
        
        blurBehind.dwFlags = DWM_BB_ENABLE | DWM_BB_TRANSITIONONMAXIMIZED;
        blurBehind.fEnable = enable;
        blurBehind.fTransitionOnMaximized = transitionOnMaximized;

        if (enable && 0 != region)
        {
            blurBehind.dwFlags |= DWM_BB_BLURREGION;
            blurBehind.hRgnBlur = region;
        }

        return ::DwmEnableBlurBehindWindow(window,
                                           &blurBehind);
    }

    template <typename T>
    void DwmSetWindowAttribute(HWND window,
                               const DWMWINDOWATTRIBUTE attribute,
                               const T& value)
    {
        ASSERT(0 != window);

        COM_VERIFY(::DwmSetWindowAttribute(window,
                                           attribute,
                                           &value,
                                           sizeof (T)));
    }

    inline void DwmSetWindowAttribute(HWND window,
                                      const DWMWINDOWATTRIBUTE attribute,
                                      bool value)
    {
        ASSERT(0 != window);

        DwmSetWindowAttribute(window,
                              attribute,
                              static_cast<BOOL>(value));
    }
}
