//*****************************************************************************
//
//  Author:         Kenny Kerr
//  Date created:   6 August 2006
//
//  Description:    Implements the image window for the sample. The image 
//                  window is where most of the effects are applied.
//
//*****************************************************************************

#pragma once

#include "BlendFunction.h"

class ImageWindow :
    public CWindowImpl<ImageWindow, CWindow, CWinTraits<WS_SYSMENU | WS_MINIMIZEBOX | WS_CLIPCHILDREN | WS_CLIPSIBLINGS, WS_EX_APPWINDOW | WS_EX_WINDOWEDGE>>
{
public:

    ImageWindow();

    BEGIN_MSG_MAP(ImageWindow)
        MESSAGE_HANDLER(WM_DWMCOMPOSITIONCHANGED, OnDwmCompositionEnabled)
        MESSAGE_HANDLER(WM_PAINT, OnPaint)
        MSG_WM_ERASEBKGND(OnEraseBackground)
        MESSAGE_HANDLER(WM_NCHITTEST, OnNcHitTest)
    END_MSG_MAP()

    void Create(DWORD extendedStyles = 0);

    //
    // Dynamic window properties
    //

    void EnableBlur(bool enable);
    void DisableTransitions(const bool disable);
    void AllowNcPaint(bool allow);
    void ExtendFrame(bool extend);
    void UpdateExtendFrameMargins(const MARGINS& margins);
    void UpdateNcRenderingPolicy(DWMNCRENDERINGPOLICY policy);
    void UpdateFlip3DPolicy(const DWMFLIP3DWINDOWPOLICY policy);
    bool SetFileName(const CString& fileName);
    void CreateLayeredWindow(bool enable);

private:

    //
    // Message handlers
    //

    LRESULT OnDwmCompositionEnabled(UINT message, WPARAM wParam, LPARAM lParam, BOOL& handled);
    LRESULT OnPaint(UINT message, WPARAM wParam, LPARAM lParam, BOOL& handled);
    bool OnEraseBackground(HDC dc);
    LRESULT OnNcHitTest(UINT message, WPARAM wParam, LPARAM lParam, BOOL& handled);

    //
    // Helper functions
    //

    void UpdateLayeredWindow();
    void EnableBlur();
    void ExtendFrame();
    void UpdateIsCompositionEnabled();

    //
    // Member variables
    //

    bool m_enableComposition;
    bool m_enableBlur;
    bool m_extendFrame;
    bool m_disableTransitions;
    DWMNCRENDERINGPOLICY m_ncRenderingPolicy;
    DWMFLIP3DWINDOWPOLICY m_flip3DPolicy;
    bool m_useLayeredWindow;
    CString m_fileName;
    MARGINS m_margins;
    bool m_isCompositionEnabled;
    CImage m_image;
    bool m_allowNcPaint;
};
