//*****************************************************************************
//
//  Author:         Kenny Kerr
//  Date created:   6 August 2006
//
//  Description:    Implements the main window for the sample. The main 
//                  window provides the "control panel" for the sample.
//
//*****************************************************************************

#pragma once

#include "resource.h"
#include "ImageWindow.h"
#include "ThumbnailWindow.h"

class MainWindow : 
    public CDialogImpl<MainWindow>,
    public CMessageFilter,
    public CIdleHandler
{
public:

    MainWindow();

    enum { IDD = IDD_MAINWINDOW };

    BEGIN_MSG_MAP(MainWindow)
        MSG_WM_CTLCOLORSTATIC(OnCtlColorStatic)
        MESSAGE_HANDLER(WM_DWMCOMPOSITIONCHANGED, OnCompositionEnabled)
        MESSAGE_HANDLER(WM_DWMCOLORIZATIONCOLORCHANGED, OnCompositionColorChanged)

        COMMAND_HANDLER(IDC_DISABLE_COMPOSITION, BN_CLICKED, OnDisableComposition)
        COMMAND_HANDLER(IDC_ENABLE_BLUR, BN_CLICKED, OnEnableBlur)
        COMMAND_HANDLER(IDC_DISABLE_TRANSITIONS, BN_CLICKED, OnDisableTransitions)
        COMMAND_HANDLER(IDC_ALLOW_NC_PAINT, BN_CLICKED, OnAllowNcPaint)
        COMMAND_HANDLER(IDC_EXTEND, BN_CLICKED, OnExtendFrame)
        COMMAND_RANGE_HANDLER(IDC_EXTEND_LEFT, IDC_EXTEND_BOTTOM, OnExtendFrameMargins)
        COMMAND_HANDLER(IDC_NC_RENDERING_POLICY, CBN_SELCHANGE, OnNcRenderingPolicy)
        COMMAND_HANDLER(IDC_FLIP3D_POLICY, CBN_SELCHANGE, OnFlip3DPolicy)
        COMMAND_HANDLER(IDC_FILENAME, EN_CHANGE, OnFileName)
        COMMAND_HANDLER(IDC_CREATE_LAYERED_WINDOW, BN_CLICKED, OnCreateLayeredWindow)
        COMMAND_HANDLER(IDC_EXCLUDE_NC_AREA, BN_CLICKED, OnThumbnailExcludeNcArea)

        COMMAND_ID_HANDLER(IDCANCEL, OnCancel)
        COMMAND_ID_HANDLER(IDC_SHOW, OnShowImageWindow)
        COMMAND_ID_HANDLER(IDC_SHOW_THUMBNAIL, OnShowThumbnail)
        NOTIFY_CODE_HANDLER(NM_CLICK, OnLink)
    END_MSG_MAP()

private:

    //
    // Virtual methods and overrides
    //

    virtual BOOL OnIdle();
    virtual void OnFinalMessage(HWND window);
    virtual BOOL PreTranslateMessage(MSG* message);

    //
    // Message handlers
    // 

    HBRUSH OnCtlColorStatic(CDCHandle dc, HWND window);
    LRESULT OnCompositionEnabled(UINT message, WPARAM wParam, LPARAM lParam, BOOL& handled);
    LRESULT OnCompositionColorChanged(UINT message, WPARAM wParam, LPARAM lParam, BOOL& handled);

    //
    // Command handlers
    //

    LRESULT OnDisableComposition(WORD notifyCode, WORD identifier, CButton checkBox, BOOL& handled);
    LRESULT OnEnableBlur(WORD notifyCode, WORD identifier, CButton checkBox, BOOL& handled);
    LRESULT OnDisableTransitions(WORD notifyCode, WORD identifier, CButton checkBox, BOOL& handled);
    LRESULT OnAllowNcPaint(WORD notifyCode, WORD identifier, CButton checkBox, BOOL& handled);
    LRESULT OnExtendFrame(WORD notifyCode, WORD identifier, CButton checkBox, BOOL& handled);
    LRESULT OnExtendFrameMargins(WORD notifyCode, WORD identifier, HWND window, BOOL& handled);
    LRESULT OnNcRenderingPolicy(WORD notifyCode, WORD identifier, CComboBox window, BOOL& handled);
    LRESULT OnFlip3DPolicy(WORD notifyCode, WORD identifier, CComboBox window, BOOL& handled);
    LRESULT OnFileName(WORD notifyCode, WORD identifier, CEdit edit, BOOL& handled);
    LRESULT OnCreateLayeredWindow(WORD notifyCode, WORD identifier, CButton checkBox, BOOL& handled);
    LRESULT OnThumbnailExcludeNcArea(WORD notifyCode, WORD identifier, CButton checkBox, BOOL& handled);

    LRESULT OnCancel(WORD notifyCode, WORD identifier, HWND window, BOOL& handled);
    LRESULT OnShowImageWindow(WORD notifyCode, WORD identifier, HWND window, BOOL& handled);
    LRESULT OnShowThumbnail(WORD notifyCode, WORD identifier, HWND window, BOOL& handled);
    LRESULT OnLink(int identifier, LPNMHDR lParam, BOOL& handled);

    //
    // Helper functions
    //

    void PrepareControls();
    void UpdateIsCompositionEnabled();
    void UpdateColorizationInfo();
    void UpdateStatus();
    void SetHelpText(int identifier, PCWSTR text);
    void CreateLinkControl();

    //
    // Member variables
    //

    ImageWindow m_imageWindow;
    ThumbnailWindow m_thumbnailWindow;
    CWindow m_statusBackground;
    CWindow m_status;
    CToolTipCtrl m_toolTips;

    bool m_isCompositionEnabled;
    Gdiplus::Color m_compositionColor;
    bool m_compositionOpaqueBlend;
};
