//*****************************************************************************
//
//  Author:         Kenny Kerr
//  Date created:   6 August 2006
//
//  Description:    Manages a DWM thumbnail registration.
//
//*****************************************************************************

#pragma once

namespace Kerr
{
    class Thumbnail
    {
    public:

        explicit Thumbnail(HTHUMBNAIL handle = 0) :
            m_handle(handle)
        {
            // Do nothing
        }

        ~Thumbnail()
        {
            if (0 != m_handle)
            {
                COM_VERIFY(Unregister());
            }
        }

        bool IsRegistered() const
        {
            return 0 != m_handle;
        }

        HRESULT Register(HWND destination,
                         HWND source)
        {
            ASSERT(0 == m_handle);

            return ::DwmRegisterThumbnail(destination,
                                          source,
                                          &m_handle);
        }

        HRESULT Unregister()
        {
            ASSERT(0 != m_handle);
            HRESULT result = ::DwmUnregisterThumbnail(m_handle);
            m_handle = 0;
            return result;
        }

        HRESULT QuerySourceSize(CSize& size)
        {
            ASSERT(0 != m_handle);

            return ::DwmQueryThumbnailSourceSize(m_handle,
                                                 &size);
        }

        HRESULT UpdateProperties(const DWM_THUMBNAIL_PROPERTIES& properties)
        {
            ASSERT(0 != m_handle);

            return ::DwmUpdateThumbnailProperties(m_handle,
                                                  &properties);
        }

    private:

        HTHUMBNAIL m_handle;
    };
}
