//*****************************************************************************
//
//  Author:         Kenny Kerr
//  Date created:   6 August 2006
//
//  Description:    Implements the thumbnail window for the sample. The 
//                  thumbnail window is used as the destination for the DWM
//                  thumbnail registration.
//
//*****************************************************************************

#include "Precompiled.h"
#include "ThumbnailWindow.h"

ThumbnailWindow::ThumbnailWindow() :
    m_source(0),
    m_excludeNcArea(false)
{
    // Do nothing
}

void ThumbnailWindow::Create(CWindow* source,
                             CRect* rect)
{
    ASSERT(0 != source);
    ASSERT(0 == m_source);
    m_source = source;

    VERIFY(__super::Create(0,
                           0,
                           L"DWM - Thumbnail"));

    VERIFY(ResizeClient(200, 200, FALSE));

    if (0 != rect)
    {
        VERIFY(SetWindowPos(0, // z-order
                            rect->left, rect->top, // location
                            rect->Width(), rect->Height(), // size
                            SWP_NOZORDER));
    }

    ShowWindow(SW_SHOWNORMAL);

    COM_VERIFY(m_thumbnail.Register(m_hWnd,
                                    *m_source));

    Update();
}

void ThumbnailWindow::Update()
{
    ASSERT(0 != m_hWnd);
    ASSERT(0 != m_source);

    DWM_THUMBNAIL_PROPERTIES properties = { 0 };
    properties.dwFlags = DWM_TNP_RECTDESTINATION | DWM_TNP_SOURCECLIENTAREAONLY;
    VERIFY(GetClientRect(&properties.rcDestination));
    properties.fSourceClientAreaOnly = m_excludeNcArea;

    m_thumbnail.UpdateProperties(properties);
}

void ThumbnailWindow::ExcludeNcArea(bool value)
{
    m_excludeNcArea = value;

    if (0 != m_hWnd)
    {
        Update();
    }
}

LRESULT ThumbnailWindow::OnCompositionEnabled(UINT /*message*/, 
                                              WPARAM /*wParam*/, 
                                              LPARAM /*lParam*/, 
                                              BOOL& /*handled*/)
{
    BOOL enabled = FALSE;
    COM_VERIFY(::DwmIsCompositionEnabled(&enabled));

    if (!enabled)
    {
        SendMessage(WM_CLOSE);
    }

    return 0;
}

bool ThumbnailWindow::OnEraseBackground(CDCHandle dc)
{
    CRect rect;
    VERIFY(GetClientRect(&rect));

    VERIFY(dc.FillRect(rect, COLOR_DESKTOP));

    return true;
}

void ThumbnailWindow::OnSize(UINT /*operation*/,
                             SIZE /*size*/)
{
    if (m_thumbnail.IsRegistered())
    {
        DWM_THUMBNAIL_PROPERTIES properties = { 0 };
        properties.dwFlags = DWM_TNP_RECTDESTINATION | DWM_TNP_SOURCECLIENTAREAONLY;
        VERIFY(GetClientRect(&properties.rcDestination));
        properties.fSourceClientAreaOnly = m_excludeNcArea;

        m_thumbnail.UpdateProperties(properties);
    }
}

void ThumbnailWindow::OnClose()
{
    m_source = 0;
    m_thumbnail.Unregister();

    VERIFY(DestroyWindow());
}
